
% Short (hah!) script to discretize a PDE and then solve it by one of two methods. 

% 0. Get rid of anything from previous runs
clear all

% 1. Get the mesh size
N = input('Number of mesh lines for the unknowns in each direction [default: 128]: ');
if isempty(N)
    N = 128;
end
n = N^2; 
disp(['Total number of unknowns is ', num2str(n)])

% 2. Create the sparse linear system Ax = b to solve
t0 = clock;
[rows, cols, vals, b] = setup(N);
t1 = clock;
setuptime = etime(t1,t0);
disp(sprintf('Time to discretize PDE: %g seconds', setuptime))

% 3. Convert the COO data structure for A to Matlab's internal format
A = sparse(rows, cols, vals);
clear rows cols vals;  % Don't need them any more

% 4. Determine which solver method to use
prompt = ['Solution method to use\n' ...
               '  1 for LU\n' ...
               '  2 for Bi-CGstab\n' ...
               '  3 for ILU(0) preconditioned Bi-CGstab\n' ...
               '  4 for ILUtp preconditioned Bi-CGstab:    '];
solver = input(prompt);

if isempty(solver)
    solver = 2;
end
tol    = 1.0e-6;
maxits = 2*N;  % More iterations may be needed for a given system.

% 5. Solve the linear system 
t0 = clock;
    [x,flag] = solvit(A, b, solver, tol, maxits);
t1 = clock;
solvetime = etime(t1,t0);
if flag ~= 0
    disp('Warning: solve of the linear system may have failed')
    disp(sprintf('On return, flag = %d', flag))
end
disp(sprintf('Time to solve PDE: %g seconds', solvetime))

% 6. Get the residual norm 
r = b - A*x;
disp(sprintf('On return, ||r|| = %g', norm(r)))

% 7. Display the solution
plotsol(N+1,N+1,x)
