
% Small driver code to show the sparsity pattern of a linear system from a
% discretized PDE on unit square/cube. Only the matrix is show, since the
% right-hand side vector g is a full 1D array usually.
%
% Try this for different sizes like N = 6, 12, 66


N = input('Number of mesh lines for the unknowns in each direction [default: 6]:  ');
% If the user did not give a value, set the default one.
if isempty(N)
    N = 6;
end

n = N*N; % order of the resulting matrix = number of unknowns

[rows, cols, vals, b] = setup(N);
% Convert to a Matlab sparse matrix
A = sparse(rows, cols, vals);

positionfig  % open a figure window
spy(A)       % show the sparsity patter
title(sprintf('A (n=%d)', n))
[L, U, p] = lu(A, 'vector');

% Can show the L and U separately ...
% positionfig
% spy(L)
% title('L')
% positionfig
% spy(U)
% title('U')

% ... but it is more common to show the L and U combined, since it is only
% the sparsity pattern that is being shown.
positionfig
spy(L+U)
title(sprintf('L+U (n=%d)', n))

